xquery version "3.0";
module namespace taricxml = "http://taricxml.fraktal.com.pl";

declare function taricxml:goods_code_trim_zeros($code as xs:string?) as xs:string {
	(:
	:)
	let $code_trimmed := replace($code,'0+$','')
	let $code_trimmed_len := string-length($code_trimmed)
	(: 
	The beginning of the code is organized into blocks of 2.
	So if our length is odd and below 4 (within the first 2 blocks), we add the 0 back to keep the integrity of the block.
	A block could be "10".
	:)
	return if ($code_trimmed_len mod 2 != 0 and $code_trimmed_len < 4) then
		string-join(($code_trimmed, "0"))
	else
		$code_trimmed
}; 


declare function taricxml:extract_description_text_and_language($goodsNomenclatureDescription_elem as node()) as node()
{
	<description lang="{$goodsNomenclatureDescription_elem/language/languageId/text()}">
		{$goodsNomenclatureDescription_elem/description/text()}
	</description>
};

declare function taricxml:extract_current_description_period($goodsNomenclature_elem as node()) as node()
{
	(
		for $desc_period in $goodsNomenclature_elem/goodsNomenclatureDescriptionPeriod
		order by $desc_period/validityStartDate/xs:dateTime(text()) descending	
		return $desc_period
	)[1]
};

declare function taricxml:extract_current_indents($goodsNomenclature_elem as node()) as xs:integer
{
  (
    for $e in $goodsNomenclature_elem/goodsNomenclatureIndents
  	order by $e/validityStartDate/xs:dateTime(text()) descending
  	return $e
	)[1]/xs:integer(numberIndents/text())
};

declare function taricxml:extract_goods_info($goodsNomenclature_elem as node()) as node()
(: 
Process a GoodsNomenclature node to get only interesting information.
Also choose the latest description and indent number
:)
{
	let $code := $goodsNomenclature_elem/goodsNomenclatureItemId/text()
	let $code_trimmed := taricxml:goods_code_trim_zeros($code)
	let $description_period_latest := taricxml:extract_current_description_period($goodsNomenclature_elem)
	let $description_by_lang := $description_period_latest/goodsNomenclatureDescription ! taricxml:extract_description_text_and_language(.)

	return <GoodsNomenclature
		code="{$code_trimmed}"
		indent="{taricxml:extract_current_indents($goodsNomenclature_elem)}"
		productline="{$goodsNomenclature_elem/produclineSuffix/text()}"
		hjid="{$goodsNomenclature_elem/hjid/text()}"
	>
		{$description_by_lang}
	</GoodsNomenclature>
};

