
(:
The provided list of goods has no explicit hierarchy, such as a parent's code, or list of children.
We reconstruct the hierarchy based on the following factors.

### The TARIC code 
The [code](https://en.wikipedia.org/wiki/TARIC_code) describes a good's place in the hierarchy.

The first two digits specify a high level category, such as *01 Live Animals* or *02 ???".
The following digits define a hierarchy of finer categories. 

Both the broad categories and specific goods are represented by nodes in the database.
A high level category will have its digits specified, followed by zeros.
For example:

* *01 Live Animals* will have the code `0100000000`
* A sub-category 

### Product line

### Indents

3.1 s to construct hierarchy
:)
declare function local:is_parent_child($parent as node(), $child as node()) as xs:boolean
{
	if (xs:integer($child/@indent) = 1+xs:integer($parent/@indent)) then
		true()
	else
		let $child_code := $child/@code/string()
		let $parent_code := $parent/@code/string()
	let $parent_code_len := string-length($parent_code)
		return (
		string-length($child_code) >= $parent_code_len 
		and 
		substring($child_code, 1, $parent_code_len) = $parent_code
	)
};


(:
@param $root: the root node whose subtree we are finding
@param $children_accumulator: the $root's children which we have found in previous interations,
	this function is recursive and accumulates only one child per call
@param $remaining_goods: the rest of the flat goods list from the original database.
@return [tree, rest]
	where tree: the subtree consisting of $root and its all descendants, organized in a hierarchy.
	rest: the $remaining_goods sequence, minus the prefix consisting of elements which were put into the constructed tree
:)
declare function local:tree_node_gather_children( $root as node(), $children_accumulator as node()*, $remaining_goods as node()*)
{
	let $next := head($remaining_goods)
	return if (not(empty($next)) and local:is_parent_child($root, $next)) then
		let $child_subtree_and_remaining_goods := local:tree_node_gather_children( $next, (), tail($remaining_goods))
		let $child_subtree := $child_subtree_and_remaining_goods(1)
		let $rem_goods_2 := $child_subtree_and_remaining_goods(2)
		return local:tree_node_gather_children($root, ($children_accumulator, $child_subtree), $rem_goods_2)
	else
		let $this_node := <GoodsTree>
			{$root/attribute::*}
			{$root/description}
			{$children_accumulator}
		</GoodsTree>
		return [ $this_node, $remaining_goods ]
};

declare function local:tree_print($g as node(), $lang as xs:string) as node()
{
	let $name := $g/description[@lang=$lang]
	return <G code="{$g/@code}" name="{$name}">
		{ $g/GoodsTree ! local:tree_print(., $lang) }
	</G>
};

declare function local:build_hierarchy($goods_flat as node()*) as node()
{
	let $goods_flat_sorted := for $g in $goods_flat order by $g/@code, $g/@productline return $g

	let $global_root := <GoodsNomenclature code="">
		<description lang="EN">Goods Nomenclature</description>
		<description lang="PL">Nomenklatura dóbr</description>
	</GoodsNomenclature>

	let $tree_and_rem := local:tree_node_gather_children($global_root, (), $goods_flat_sorted)
	let $tree := $tree_and_rem(1)
	let $rem := $tree_and_rem(2)

	return if (empty($rem)) then
		$tree
	else
		<Error>
			<message>Remaining unprocessed nodes {count($rem)}</message>
			{$rem}
		</Error>
};


let $tree := (
	let $goods_flat := doc("taric/goods_nomenclature_flat.xml")/GoodsNomenclatureFlat/GoodsNomenclature
	return local:build_hierarchy($goods_flat)
)
let $tree_readable := local:tree_print($tree, "EN")

return (
	file:write("../data_out/goods_nomenclature_tree.xml", $tree),
	file:write("../data_out/goods_nomenclature_tree_readable_en.xml", $tree_readable),
	db:replace("taric-base", "goods_nomenclature_tree.xml", $tree)
	(: db:create would erase whe whole db, not just the specified xml file :)
)
